/*******************************************************************************
 * Copyright (c) 2000, 2007 IBM Corporation and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *     IBM Corporation - initial API and implementation
 * Port to the D programming language:
 *     Frank Benoit <benoit@tionex.de>
 *******************************************************************************/
module dwt.custom.MovementEvent;

import dwt.events.TypedEvent;
import dwt.custom.StyledTextEvent;
import dwt.dwthelper.utils;

/**
 * This event is sent when a new offset is required based on the current
 * offset and a movement type.
 *
 * @since 3.3
 */
public class MovementEvent : TypedEvent {

    /**
     * line start offset (input)
     */
    public int lineOffset;

    /**
     * line text (input)
     */
    public String lineText;

    /**
     * the current offset (input)
     */
    public int offset;

    /**
     * the new offset  (input, output)
     */
    public int newOffset;

    /**
     * the movement type (input)
     *
     * @see dwt.DWT#MOVEMENT_WORD
     * @see dwt.DWT#MOVEMENT_WORD_END
     * @see dwt.DWT#MOVEMENT_WORD_START
     * @see dwt.DWT#MOVEMENT_CHAR
     * @see dwt.DWT#MOVEMENT_CLUSTER
     */
    public int movement;

    static final long serialVersionUID = 3978765487853324342L;

public this(StyledTextEvent e) {
    super(cast(Object)e);
    lineOffset = e.detail;
    lineText = e.text;
    movement = e.count;
    offset = e.start;
    newOffset = e.end;
}
}


